<?php

namespace Drupal\google_pubsub\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\google_pubsub\Services\PubSubServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for the Google PubSub Subscription.
 */
class PubSubSubscriptionController extends ControllerBase {

  /**
   * The PubSub service.
   *
   * @var \Drupal\google_pubsub\Services\PubSubServiceInterface
   */
  protected $pubSubService;

  /**
   * Constructs a PubSubSubscriptionController object.
   *
   * @param \Drupal\google_pubsub\Services\PubSubServiceInterface $pubSubService
   *   The PubSub service.
   */
  public function __construct(PubSubServiceInterface $pubSubService) {
    $this->pubSubService = $pubSubService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('google_pubsub.pub_sub_service')
    );
  }

  /**
   * Displays the list of subscriptions for a topic.
   *
   * @param string $topicName
   *   The name of the topic.
   */
  public function subscriptions($topicName) {
    $project_id = $this->config('google_pubsub.settings')->get('project_id');
    $topic = 'projects/' . $project_id . '/topics/' . $topicName;
    $subscriptions = $this->pubSubService->listSubscriptions($topic);
    $subscriptions_names = [];
    foreach ($subscriptions as $subscription) {
      $info = $subscription->info();
      if ($info['topic'] == $topic) {
        $subscriptions_names[] = str_replace('projects/' . $project_id . '/subscriptions/', '', $subscription->name());
      }
    }
    return [
      '#theme' => 'google_pubsub_subscriptions',
      '#subscriptions' => $subscriptions_names,
      '#topic_name' => $topic,
    ];
  }

  /**
   * Provides the title for the subscriptions page.
   *
   * @param string $topicName
   *   The name of the topic.
   *
   * @return string
   *   The title of the page.
   */
  public function subscriptionsTitle($topicName) {
    return $this->t('Google PubSub Subscriptions for @topicName', ['@topicName' => urldecode($topicName)]);
  }

  /**
   * Displays the list of messages for a subscription.
   *
   * @param string $subscriptionName
   *   The name of the subscription.
   */
  public function messages($subscriptionName) {
    $messages = $this->pubSubService->pullMessages($subscriptionName, 10);
    return [
      '#theme' => 'google_pubsub_messages',
      '#messages' => $messages,
      '#subscription_name' => $subscriptionName,
    ];
  }

}
